package com.govmade.modules.system.controller;

import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.govmade.common.utils.PageTreeUtils;
import com.govmade.common.utils.R;
import com.govmade.common.validator.Assert;
import com.govmade.modules.basic.controller.AbstractController;
import com.govmade.modules.system.entity.DictEntity;
import com.govmade.modules.system.service.DictService;

/**
 * 系统管理 - 字典设置
 * 
 * @author Fred
 * @email fangtaosh@qq.com
 * @date 2018年8月7日
 */
@RestController
@RequestMapping("/system/dict")
public class DictController extends AbstractController {

	@Autowired
	private DictService dictService;


	/**
	 * 用户列表
	 */
	@GetMapping("/list")
	public R list(Map<String, Object> params) {
		
		List<DictEntity> dictList = dictService.queryChilds("city_level");
		
		return R.ok().put("childs", dictList);
	}
	
	/**
	 * 所有字典数据树结构
	 */
	@GetMapping("/listTree")
	public R listTree(@RequestParam Map<String, Object> params) {
		PageTreeUtils page = dictService.queryPage(params);
		return R.ok().put("page", page);
	}
	
	
	/**
	 * 保存或更新字典
	 */
	@PostMapping("/save")
	public R save(DictEntity dict) {

		if (null == dict.getId()) {
			if (this.dictService.checkDict(dict.getName()) > 0) {
				return R.error(dict.getName() + " 已存在!");
			}
		}
		dictService.save(dict);
		return R.ok();
	}
/**
	 * 批量删除字典
	 * 
	 * @param ids
	 * @return
	 */
	@DeleteMapping("/delete")
	public R deleteDict(@RequestBody Set<Long> ids) {
		Assert.isNull(ids, "删除项不能为空");

		dictService.deleteBatch(ids);
		return R.ok();
	}

}
