package com.govmade.modules.system.controller;

import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.govmade.common.utils.PageUtils;
import com.govmade.common.utils.R;
import com.govmade.common.validator.Assert;
import com.govmade.modules.basic.controller.AbstractController;
import com.govmade.modules.system.entity.ModuleEntity;
import com.govmade.modules.system.service.ModuleService;

import io.swagger.annotations.ApiOperation;

/**
 * 系统管理 - 模块设置
 * 
 * @author 刘弈臻
 * @date 2018年8月9日
 */
@RestController
@RequestMapping("/system/module")
public class ModuleController extends AbstractController{
	
	@Autowired
	private ModuleService moduleService;
	
	/**
	 * 模块列表
	 */
	@GetMapping("/list")
	public R list(@RequestParam Map<String, Object> params) {
		
		PageUtils page = moduleService.queryPage(params);
		return R.ok().put("page", page);
	}
	
	@GetMapping("moduleList")
    public R moduleList() {
        List<ModuleEntity> modulelist = moduleService.queryModuleList();
        return R.ok().put("data", modulelist);
    }
	
	/**
	 * 保存或更新模块
	 */
	@PostMapping("/save")
	public R save(ModuleEntity module) {

		if (null == module.getId()) {
			if (this.moduleService.checkModule(module.getName()) > 0) {
				return R.error(module.getName() + " 已存在!");
			}
			
		}
		moduleService.save(module);
		return R.ok();
	}
	
	/**
	 * 批量删除模块
	 * 
	 * @param ids
	 * @return
	 */
	@DeleteMapping("/delete")
	public R deleteModule(@RequestBody Set<Long> ids) {
		Assert.isNull(ids, "删除项不能为空");

		moduleService.deleteBatch(ids);
		return R.ok();
	}
	
}
